## Parameters
sif.file = "specify_path_to_file"
vcf.file = "specify_path_to_file"
phased = FALSE ## TRUE if genotypes in VCF format are phased
out.dir = "specify_path_to_dir"
model.name = "specify_model_name"
call.rate = 1 ## fraction of samples with genotype calls for a specific SNP

## Load SIF
sif = read.delim(sif.file,sep="\t",as.is=T)
sif$Gender[which(sif$Gender=="male")] = 1
sif$Gender[which(sif$Gender=="female")] = 2

## Load VCF file
tmp = read.table(vcf.file,as.is=T,header=F,sep="\t",comment.char="#")

## select only variants with single reference and alternative bases
l1 = sapply(tmp[,4],function(x) length(strsplit(x,"")[[1]]))
l2 = sapply(tmp[,5],function(x) length(strsplit(x,"")[[1]]))
tmp = tmp[which(l1==1&l2==1),]

## when MAF specified select only if MAF>=1%
maf = as.numeric(sapply(tmp[,8],function(x) 
{
  elems = strsplit(x,";")[[1]]
  elems = elems[grep("MAF=",elems)]
  if(length(elems)==1)
  {
    vals = strsplit(elems[1],"=")[[1]][2]
    vals = as.numeric(strsplit(vals,",")[[1]][1])
    if(is.na(vals))
      vals = 1
    return(vals)
  }
  return(1)
}))
tmp = tmp[which(maf>=0.01),]

## Create out files
geno = as.matrix(tmp[,10:ncol(tmp)])
if(phased)
{
  ped.tmp = apply(geno,2,function(x) gsub(".\\|.","0 0",gsub("1\\|1","2 2",gsub("0\\|0","1 1",gsub("1\\|0","1 2",gsub("0\\|1","1 2",x))))))
} else
{
  ped.tmp = apply(geno,2,function(x) gsub("./.","0 0",gsub("1/1","2 2",gsub("0/0","1 1",gsub("0/1","1 2",x)))))
}

if(!all(ped.tmp%in%c("0 0","1 1","1 2","2 2")))
{
  cat("VCF format not valid!!!\n")
  q()
}

retain = which(apply(ped.tmp,1,function(x) length(which(x!="0 0"))/length(x))>=call.rate)
if(length(retain)>0)
{
  ped.tmp = ped.tmp[retain,]
} else
{
  cat("No SNPs to retain after call rate filtering!!!\n")
  q()
}

ped.tmp = t(ped.tmp)
rownames(ped.tmp) = header

ped = ped.tmp
vcf = cbind(tmp[,1:5],rep(".",nrow(tmp)),rep(".",nrow(tmp)),rep(".",nrow(tmp)))
colnames(vcf) = colnames(tmp)[1:8]
map = cbind(tmp[,1],tmp[,3],rep("0",nrow(tmp)),tmp[,2])
colnames(map) = c("chr","rs","gd","bp")

ped = cbind(1:nrow(ped),header,rep(0,nrow(ped)),rep(0,nrow(ped)),sif$Gender,rep(1,nrow(ped)),ped)

write.table(ped,paste(out.dir,"/",model.name,".Model.ped",sep=""),sep="\t",row.names=F,col.names=F,quote=F)
write.table(vcf,paste(out.dir,"/",model.name,".Model.vcf",sep=""),sep="\t",row.names=F,col.names=F,quote=F)
write.table(map,paste(out.dir,"/",model.name,".Model.map",sep=""),sep="\t",row.names=F,col.names=F,quote=F)

