## Refinement analysis
cmd_args <- commandArgs(trailingOnly = T)

verbose = FALSE
run.genotype = TRUE
reduce.composite.model = FALSE
composite.model.call.rate = 1
sif.file = ""

## Load configuration file
configFile <- cmd_args[1]
source(configFile)
source(paste(source.dir,"Functions.R",sep=""))

## Load samples list
bam.files = unique(as.character(read.delim(bam.list,header=F)[,1]))
sample.names = gsub(".bam","",basename(bam.files))

## Check subsets consistency
if(model!="")
{
  res = get.Model(model,ped.lab,vcf.lab,map.lab)
  sif.file = res[[4]]
  vcf.file = res[[3]]
} else
{
  if(sif.file==""|vcf.file=="")
    error.message("Reference model files are missing.\n")
}

sif = read.table(sif.file,sep="\t",as.is=T,header=T)
races = unique(sif$Race)
for(s in 1:length(subsets))
{
  if(all(subsets[[s]]%in%races))
  {
    races = subsets[[s]]
  } else
  {
    error.message("Subsets description not valid.\n")
  }
}

## Execute EthSEQ first inference
command = paste("Rscript ",source.dir,"EthSEQ.R ",configFile,sep="")
system(command)
eigenstrat.dir = paste(out.dir,"EIGENSTRAT/",sep="")

## Execute multi-steps inference
for(s in 1:length(subsets))
{
  ## Load PED
  ped = read.table(paste(eigenstrat.dir,"DatasetReference.ped",sep=""),sep="\t",as.is=T,header=F)
  ## Load Report
  report = read.delim(paste(eigenstrat.dir,"Report.csv",sep=""),as.is=T,header=T,sep="\t")
  ## Create new folders
  command = paste("mkdir ",out.dir,paste(subsets[[s]],collapse="_"),sep="")
  system(command)
  command = paste("mkdir ",out.dir,paste(subsets[[s]],collapse="_"),"/EIGENSTRAT/",sep="")
  system(command)
  ## Select samples of specified ethnicities and generate reference and target models
  samples = report$sample[grep(paste(subsets[[s]],collapse="|"),report$ethnicity)]
  ped.tmp = ped[which(ped[,2]%in%samples),]
  write.table(ped.tmp,paste(out.dir,"TMP_TARGET.ped",sep=""),sep="\t",row.names=F,quote=F,col.names=F)
  samples = sif$Sample[which(sif$Race%in%subsets[[s]])]
  ped.tmp = ped[which(ped[,2]%in%samples),]
  write.table(ped.tmp,paste(out.dir,"TMP_REFERENCE.ped",sep=""),sep="\t",row.names=F,quote=F,col.names=F)
  ## Reduce samples list
  write(report$bam[grep(paste(subsets[[s]],collapse="|"),report$ethnicity)],paste(out.dir,paste(subsets[[s]],collapse="_"),"/List.txt",sep=""))
  ## Generate new EthSEQ config file
  config = paste(
    "source.dir = \"",source.dir,"\"\n",
    "bam.list = \"",out.dir,paste(subsets[[s]],collapse="_"),"/List.txt\"","\n",
    "out.dir = \"",out.dir,paste(subsets[[s]],collapse="_"),"/\"","\n",
    "eigenstrat.path = \"",eigenstrat.path,"\"\n",
    "model = \"\"","\n",
    "ASEQ.path = \"",ASEQ.path,"\"\n",
    "mbq=",mbq,"\n",
    "mrq=",mrq,"\n",
    "mdc=",mdc,"\n",
    "cores=",cores,"\n",
    "verbose=",verbose,"\n",
    "run.genotype=FALSE","\n",
    "reduce.composite.model = TRUE","\n",
    "model.ped = \"",out.dir,"TMP_REFERENCE.ped","\"\n",
    "ped.target = \"",out.dir,"TMP_TARGET.ped","\"\n",
    "model.map = \"",eigenstrat.dir,"DatasetReference.map","\"\n",
    "vcf.file = \"",vcf.file,"\"\n",
    "sif.file = \"",sif.file,"\"",sep="")
  write(config,paste(out.dir,paste(subsets[[s]],collapse="_"),"/EthSEQ.Config.R",sep=""))
  command = paste("Rscript ",source.dir,"EthSEQ.R ",out.dir,paste(subsets[[s]],collapse="_"),"/EthSEQ.Config.R",sep="")
  system(command)
  eigenstrat.dir = paste(out.dir,paste(subsets[[s]],collapse="_"),"/EIGENSTRAT/",sep="")
}

### Create fianl annotation

eigenstrat.dir = paste(out.dir,"EIGENSTRAT/",sep="")
report = read.delim(paste(eigenstrat.dir,"Report.csv",sep=""),as.is=T,header=T,sep="\t")
report = report[,1:3]

for(s in 1:length(subsets))
{
  eigenstrat.dir = paste(out.dir,paste(subsets[[s]],collapse="_"),"/EIGENSTRAT/",sep="")
  tmp = read.delim(paste(eigenstrat.dir,"Report.csv",sep=""),as.is=T,header=T,sep="\t")
  cat(all(report$sample[which(report$sample%in%tmp$sample)]==tmp$sample))
  report$ethnicity[which(report$sample%in%tmp$sample)] = tmp$ethnicity
}

system(paste("rm ",out.dir,"/TMP_TARGET.ped",sep=""))
system(paste("rm ",out.dir,"/TMP_REFERENCE.ped",sep=""))
write.table(report,paste(out.dir,"/RefinedReport.csv",sep=""),sep="\t",quote=F,row.names=F)


